package com.diagbot.facade;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.client.UserServiceClient;
import com.diagbot.dto.RelationContactListDTO;
import com.diagbot.dto.RelationNodeDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.Concept;
import com.diagbot.entity.Relation;
import com.diagbot.entity.RelationOrder;
import com.diagbot.enums.LexiconTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.RelationService;
import com.diagbot.service.impl.RelationOrderServiceImpl;
import com.diagbot.service.impl.RelationServiceImpl;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.RelationContactDetailVO;
import com.diagbot.vo.RelationContactListVO;
import com.diagbot.vo.RelationNodeVO;
import com.diagbot.vo.RemoveRelationContactVO;

/**
 * @Description: 医学术语关联业务层
 * @author: Weixuan Huang
 * @time: 2019/1/16 14:17
 */
@Component
public class RelationContactFacade extends RelationServiceImpl {
	
	@Autowired
    private ConceptFacade conceptFacade; 
	@Autowired
    private RelationOrderFacade relationOrderFacade; 
	@Autowired
    @Qualifier("relationServiceImpl")
    private RelationService relationService;
	@Autowired
    @Qualifier("relationOrderServiceImpl")
    private RelationOrderServiceImpl relationOrderServiceImpl;
	@Autowired
    private UserServiceClient userServiceClient;
	
    /**
     * 医学术语关联维护-列表
     * @param relationContactListVO
     * @return
     */
    public IPage<RelationContactListDTO> relationContactList(RelationContactListVO relationContactListVO) {
    	IPage<RelationContactListDTO> ipage = this.baseMapper.relationContactList(relationContactListVO);
        if (ipage.getRecords().size() > 0) {
            List<String> ids = ipage.getRecords().stream().map(i -> i.getOperName()).distinct().collect(Collectors.toList());
            RespDTO<Map<String, String>> respDTO = userServiceClient.getUserInfoByIds(ids);
            RespDTOUtil.respNGDealCover(respDTO, "获取用户信息失败");
            ipage.getRecords().forEach(i -> {
                i.setOperName(respDTO.data.get(i.getOperName()));
            });
        }
        return ipage;
    }

	/**
	 * 医学术语关联维护/医学术语多层关联维护/化验子项维护-添加或者编辑
	 * @param relationNodeVO
	 * @return
	 */
	public Boolean addRelation(RelationNodeVO relationNodeVO) {
		if(relationNodeVO.getConceptId()==null){
			throw new CommonException(CommonErrorCode.RPC_ERROR,"conceptId必填！");
		}
		if(relationNodeVO.getNodeList()==null||relationNodeVO.getNodeList().size()==0){
			throw new CommonException(CommonErrorCode.RPC_ERROR,"nodeList不能为空！");
		}
		
		List<Long> relationIdList = repairRelationDataForDelBeforeAdd(relationNodeVO.getConceptId(), relationNodeVO.getSonRelationId());
		if(ListUtil.isNotEmpty(relationIdList)){
			QueryWrapper<RelationOrder> relationOrderQe = new QueryWrapper<>();
			relationOrderQe.in("t_relation_id", relationIdList);
			relationOrderFacade.remove(relationOrderQe);
		}
		
		List<List<Relation>> relationGroupList = repairRelationDataForAdd(relationNodeVO);
		String currentUser = UserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();
        List<Relation> relationList = new ArrayList<>();
        relationGroupList.forEach(i->{
        	i.forEach(j->{
        		j.setCreator(currentUser);
            	j.setGmtCreate(now);
            	j.setModifier(currentUser);
            	j.setGmtModified(now);
            	relationList.add(j);
        	});
        });
        relationService.saveOrUpdateBatch(relationList);
        
        if(relationNodeVO.getIsOrderBy()==1){
        	List<RelationOrder> relationOrderList = new ArrayList<>();
        	relationGroupList.forEach(i->{
        		int orderNo = 0;
            	for(Relation j : i){
            		orderNo++;
            		RelationOrder relationOrder = new RelationOrder();
            		relationOrder.setOrderNo(orderNo);
            		relationOrder.settRelationId(j.getId());
            		relationOrder.setGmtCreate(now);
            		relationOrder.setGmtModified(now);
            		relationOrder.setCreator(currentUser);
            		relationOrder.setModifier(currentUser);
            		relationOrderList.add(relationOrder);
            	}
            });
        	relationOrderServiceImpl.saveBatch(relationOrderList);
        }
        
		return true;
	}
	
	/**
	 * 处理关系节点-添加或者编辑前先删除掉
	 * @param conceptId
	 * @param relationId
	 * @return
	 */
	private List<Long> repairRelationDataForDelBeforeAdd(Long conceptId,Long relationId){
		List<Long> relationIdList = new ArrayList<>();
		
		QueryWrapper<Relation> relationQe = new QueryWrapper<>();
		relationQe.eq("start_id", conceptId);
		relationQe.eq(relationId!=null,"relation_id", relationId);
		List<Relation> relationList = list(relationQe);
		
		if(relationList.size()==0){
			return relationIdList;
		}
		
		relationIdList.addAll(relationList.stream().map(i->i.getId()).collect(Collectors.toList()));
		
		relationList.forEach(i->{
			relationIdList.addAll(repairRelationDataForDelBeforeAdd(i.getEndId(),relationId));
		});
		
		if(!remove(relationQe)){
			throw new CommonException(CommonErrorCode.SERVER_IS_ERROR);
		}
		
		return relationIdList;
	}
	
	/**
	 * 处理关系节点-添加或者编辑事件
	 * @param relationNodeVO
	 * @return
	 */
	private List<List<Relation>> repairRelationDataForAdd(RelationNodeVO relationNodeVO){
		List<List<Relation>> retList = new ArrayList<>();
		
		if(relationNodeVO.getNodeList()!=null&&relationNodeVO.getNodeList().size()>0){
			List<Relation> relationList = new ArrayList<>();
			relationNodeVO.getNodeList().forEach(i->{
				Relation relation = new Relation();
				relation.setStartId(relationNodeVO.getConceptId());
				relation.setEndId(i.getConceptId());
				relation.setRelationId(i.getRelationId());
				relationList.add(relation);
				
				if(i.getNodeList()!=null&&i.getNodeList().size()>0){
					retList.addAll(repairRelationDataForAdd(i));
				}
			});
			retList.add(relationList);
		}
		
		return retList;
	}
	
	/**
	 * 医学术语关联维护/医学术语多层关联维护/化验子项维护-详情
	 * @param relationContactDetailVO
	 * @return
	 */
	public RelationNodeDTO relationContactDetail(RelationContactDetailVO relationContactDetailVO){
		RelationNodeDTO relationNodeDTO = new RelationNodeDTO();
		
		Concept concept = conceptFacade.getById(relationContactDetailVO.getConceptId());
		relationNodeDTO.setConceptId(concept.getId());
		relationNodeDTO.setConceptName(concept.getLibName());
		relationNodeDTO.setConceptTypeId(concept.getLibType());
		relationNodeDTO.setConceptTypeName(LexiconTypeEnum.getName(concept.getLibType().intValue()));
		relationNodeDTO.setConceptNameType(concept.getLibName()+"("+LexiconTypeEnum.getName(concept.getLibType().intValue())+")");
		
		relationNodeDTO.setNodeList(repairRelationDataForQuery(relationContactDetailVO.getConceptId(), relationContactDetailVO.getRelationId(),null));
		
		return relationNodeDTO;
	}
	
	/**
	 * 处理关系节点-查询事件
	 * @param conceptId
	 * @param relationId
	 * @param hookConceptIds
	 * @return
	 */
	private List<RelationNodeDTO> repairRelationDataForQuery(Long conceptId,Long relationId,List<Long> hookConceptIds){
		if(hookConceptIds==null){
			hookConceptIds = new ArrayList<>();
			hookConceptIds.add(conceptId);
		}
		
		List<RelationNodeDTO> relationNodeDTOList = this.baseMapper.getRelationNodeDTOs(conceptId,relationId);
		
		for(RelationNodeDTO i : relationNodeDTOList){
			i.setConceptTypeName(LexiconTypeEnum.getName(i.getConceptTypeId().intValue()));
			i.setConceptNameType(i.getConceptName()+"("+LexiconTypeEnum.getName(i.getConceptTypeId().intValue())+")");
			i.setParentConceptTypeName(LexiconTypeEnum.getName(i.getParentConceptTypeId().intValue()));
			i.setParentConceptNameType(i.getParentConceptName()+"("+LexiconTypeEnum.getName(i.getParentConceptTypeId().intValue())+")");
			
			if(!hookConceptIds.contains(i.getConceptId())){
				List<Long> hookConceptIds_ = new ArrayList<>();
				hookConceptIds_.addAll(hookConceptIds);
				hookConceptIds_.add(i.getConceptId());
				i.setNodeList(repairRelationDataForQuery(i.getConceptId(), relationId,hookConceptIds_));
			}
		}
		
		return relationNodeDTOList;
	}
	
	/**
	 * 化验子项维护-逻辑删除或者恢复
	 * @param removeRelationContactVO
	 * @return
	 */
	public Boolean removeRelationContact(RemoveRelationContactVO removeRelationContactVO){
		String currentUser = UserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();
        repairRelationDataForRemove(removeRelationContactVO.getConceptId(), removeRelationContactVO.getRelationId(),removeRelationContactVO.getIsDeleted(),currentUser,now);
		return true;
	}
	
	/**
	 * 化验子项维护-逻辑删除或者恢复-处理关系节点
	 * @param conceptId
	 * @param relationId
	 * @param isDeleted
	 * @param currentUser 操作人
	 * @param now 操作时间
	 */
	private void repairRelationDataForRemove(Long conceptId,Long relationId,String isDeleted,String currentUser,Date now){
		QueryWrapper<Relation> relationQe = new QueryWrapper<>();
		relationQe.eq("start_id", conceptId);
		relationQe.eq(relationId!=null,"relation_id", relationId);
		List<Relation> relationList = list(relationQe);
		
		if(relationList.size()==0){
			return;
		}
		
		if(relationList.stream().map(i->i.getIsDeleted()).distinct().count()>1){
			throw new CommonException(CommonErrorCode.RPC_ERROR,"数据异常！");
		}
		
		String isDeleted_ = relationList.get(0).getIsDeleted();
		if(isDeleted.equals(isDeleted_)){
    		if(isDeleted_.equals("Y")){
    			throw new CommonException(CommonErrorCode.RPC_ERROR,"该数据已删除！");
    		}
    		if(isDeleted_.equals("N")){
    			throw new CommonException(CommonErrorCode.RPC_ERROR,"该数据已恢复！");
    		}
    	}
		
		Relation relation = new Relation();
		relation.setIsDeleted(isDeleted);
		relation.setGmtModified(now);
		relation.setModifier(currentUser);
		if(!update(relation, relationQe)){
			throw new CommonException(CommonErrorCode.SERVER_IS_ERROR);
		}
		
		relationList.forEach(i->{
			repairRelationDataForRemove(i.getEndId(), relationId,isDeleted,currentUser,now);
		});
	}
	
	/**
	 * 医学术语关联维护/医学术语多层关联维护-物理删除
	 * @param removeRelationContactVO
	 * @return
	 */
	public Boolean deleteRelationContact(RemoveRelationContactVO removeRelationContactVO){
		repairRelationDataForDelete(removeRelationContactVO.getConceptId(), removeRelationContactVO.getRelationId());
		return true;
	}
	
	/**
	 * 医学术语关联维护/医学术语多层关联维护-物理删除-处理关系节点
	 * @param conceptId
	 * @param relationId
	 */
	private void repairRelationDataForDelete(Long conceptId,Long relationId){
		QueryWrapper<Relation> relationQe = new QueryWrapper<>();
		relationQe.eq("start_id", conceptId);
		relationQe.eq(relationId!=null,"relation_id", relationId);
		List<Relation> relationList = list(relationQe);
		
		if(relationList.size()==0){
			return;
		}
		
		if(!remove(relationQe)){
			throw new CommonException(CommonErrorCode.SERVER_IS_ERROR);
		}
		
		relationList.forEach(i->{
			repairRelationDataForDelete(i.getEndId(), relationId);
		});
	}

}
